function limits = disturbance_set_function(theta_error,v,p,distur_limit_pos,distur_limit_dir,sin_max,cos_max)
%{
This function caluclates disturbance given current state 
:vector theta_error: lowest and highest error in direction [rad]
:vector v: nominal inputs i.e. nominal velocity and nominal angular velocity
:float p: radius of the robot
:float distur_limit_pos: maximum error in linear velocity
:float distur_limit_dir: maximum error in angular velocity
:float sin_max/cos_max: maximum value of the sine and the cosine function
of direction

:matrix return: limits of disturbance
%}

limits = zeros(3,2);
limits(1:2,:)=[-distur_limit_pos*cos_max,distur_limit_pos*cos_max;-distur_limit_pos*sin_max,distur_limit_pos*sin_max];
sin_limit = [-sin(theta_error(1))+theta_error(1);-sin(theta_error(2))+theta_error(2)]/p;
if v(1)>0
    sin_limit=sin_limit*v(1);
else
    sin_limit=flip(sin_limit)*v(1);
end
cos_limit = [cos(max(abs(theta_error)))-1;0];
if v(2)>0
    cos_limit=cos_limit*v(2);
else
    cos_limit=flip(cos_limit)*v(2);
end
theta_limit=sin_limit+cos_limit;


%limits(3,:) = [-theta_limit,theta_limit];
limits(3,:) = theta_limit'+[-distur_limit_dir,distur_limit_dir];

end
