function out=get_SDDTMPC_input_for_python(state)
%{
This function is called by Python program to solve SDDTMPC given the
current state.
:vector state: current state
:cell return: The cell vector contains the computed input and exitflag that
shows if the input was found without any problems or not.
%}



disp('start matlab')
state=state'; %Python cannot traspose its 'lists' so it has to be done here
cd('tbxmanager')
tbxmanager restorepath
mpt_init
cd('..')
a=0.46;% max speed
p=0.4731/2; % robot's radius 
b=a/p; % maximum angular velocity
n_dist = 0.2*a; % Distrbance in linear velocity
n_direction = 0.2*b; %Disturbance in angular velocity
% v_r and w_r are used to construct leader's trajectory and pd is distance
% to the leader

% MPC parameters
N=20; % horizon
%max_iterations=length(path)-N-1;
Ts=0.2; % sampling time
Q=eye(2)*0.4; % state cost 
R=0.2; % input cost
K=eye(2); % ancilary control law constant
terminal_set_size=0.5;
sys_helper = c2d(ss(-K(1),[],[],[]),Ts);
tube_size = n_dist*Ts/(1-sys_helper.A);%0.55*n_dist;%a/k*(lambda_tube);%;
lambda_tube= (sqrt(2)/2-sqrt(2)*n_dist/a*Ts)*0.4; %input tube constant

max_size=Ts*N*a*lambda_tube+tube_size+terminal_set_size; %Max size of the environment
dist_set_func = @(theta_err,v,max_sin,max_cos) disturbance_set_function(theta_err,v,p,n_dist,n_direction,max_sin,max_cos); 




%% NTMPC - preperation 
% fmincon will find such input u that E*u<G. 
% If U is a vector of u of the horizon then A_fmincon*U<b_mincon
% U also has the output state at the end.
A_omega = [1,1;-1,1;1,-1;-1,-1];
b_omega = ones(4,1)*terminal_set_size;

E=[1/a,1/b;
	-1/a,1/b;
	1/a,-1/b;
	-1/a,-1/b];
G=ones(4,1)*lambda_tube;
options = optimoptions('fmincon','Algorithm','interior-point','Display','none','MaxFunctionEvaluations',10^6);
[A_fmincon,b_fmincon] = get_prediction_constraints(E,G,N);
A_fmincon=[A_fmincon,zeros(N*4,3);zeros(6,N*2),[1/tube_size,0,0;-1/tube_size,0,0;0,1/tube_size,0;0,-1/tube_size,0;0,0,1/pi;0,0,-1/pi]];
b_fmincon=[b_fmincon;ones(6,1)];
 
%% Parameters
% Robot's parameters

    shape = get_tube_of_full_robot([0,0],p,state(1:2)');
    shape.minVRep();
    body_outside_of_environment_flag=false;
    for i=1:8
        if norm(shape.V(i,:))>max_size
            body_outside_of_environment_flag=true;
            break
        end
    end
    
    if body_outside_of_environment_flag
        error('body outside of environment')
    end
    disp("There is no point outside")	

    %SDDTMPC simulation
    % Get warm start
    fmincon_state=state;
    cost_fun_fmincon = @(u) fmincon_cost_function(u,fmincon_state,N,Q,R,p,Ts);
    nonlinear_model =@(x,u) discrete_model(x',u,p,Ts);
    terminal_constraint_func= @(u) fmincon_termianl_constraint(u,fmincon_state,nonlinear_model,N,2,A_omega,b_omega,p+tube_size*2,max_size);
    [inputs,fval,exitflag,output] = fmincon(cost_fun_fmincon,zeros(2*N+3,1),A_fmincon,b_fmincon,[],[],[],[],terminal_constraint_func,options);
    if exitflag<1
        error('fmincon_failure')
    end

    %Solve SDDTMPC
    tic
    [inputs,fval,exitflag,message]  = SDDTMPC_solver(cost_fun_fmincon,inputs,state,...
       nonlinear_model,K,p,[-a;-b],[a;b],E,G/lambda_tube,tube_size,terminal_set_size,N,dist_set_func,150,Ts,max_size);
    if fval>10000
        error('SDDTMPC failure')
    end
    toc
    mv=inputs(1:2);% Get nominal input

    error_val = -inputs(end-2:end);
    nominal_state=-error_val+state;
    % Store data and update state
    anciallary_u=rot(state(3),p)^(-1)*(rot(nominal_state(3),p)*mv-K*error_val(1:2)); %real input
    out={anciallary_u,exitflag};

end

function cost=myCostFunction(X,U,horizon,Q,R,p)  
    cost=0;
    for i = 1:horizon
        if i==horizon
            Q=0.5;
        end
        p_e= X(i,1:2)';
        cost=cost+p_e'*Q*p_e;
        u_e=[U(i,1);
            p*U(i,2)];
        cost = cost + u_e'*R*u_e;

    end

end


function cost=fmincon_cost_function(u,x,horizon,Q,R,p,Ts)
    x=x+u(end-2:end);
    All_U = reshape(u(1:end-1),(length(u)-3)/horizon,horizon+1)';
    X=zeros(horizon,length(x));
    for i=1:horizon
        x = discrete_model(x',All_U(i,:),p,Ts);
        X(i,:)=x;
    end
    cost=myCostFunction(X,All_U,horizon,Q,R,p);
end


function [c,ceq]=fmincon_termianl_constraint(inputs,nominal_state,model,horizon,number_of_inputs,A_omega,b_omega,robot_size,world_size)
    c=zeros(horizon+4,1);

    nominal_state=nominal_state+inputs(end-2:end);
    for i=1:horizon
        v=inputs((number_of_inputs*(i-1)+1):number_of_inputs*i)';
        nominal_state=model(nominal_state,v);
        dist =norm(nominal_state(1:2));
        c(i) = dist-world_size+robot_size;
    end
    c(end-3:end)= A_omega*nominal_state(1:2)-b_omega;
    ceq=0;
end
