function main(pointer)

filename =strcat('dataset_final',num2str(pointer)); %Create file where the data will be saved
filename_helper = strcat(filename,'helper'); %A copy of the data in case the original data are damaged
dataset=zeros(1000000,9);
a=0.46;% max speed
p=0.4731/2; % robot's radius 
b=a/p; % maximum angular velocity
%n=0.004; % maximum distance disturbance affecting follower
n_dist = 0.2*a;
n_direction = 0.2*b;
% v_r and w_r are used to construct leader's trajectory and pd is distance
% to the leaderx

% MPC parameters
N=20; % horizon
%max_iterations=length(path)-N-1;
Ts=0.2; % sampling time
Q=eye(2)*0.4; % state cost 
R=0.2; % input cost
K=eye(2); % ancilary control law constant
terminal_set_size=0.5;
sys_helper = c2d(ss(-K(1),[],[],[]),Ts);
tube_size = n_dist*Ts/(1-sys_helper.A);%0.55*n_dist;%a/k*(lambda_tube);%;
lambda_tube= (sqrt(2)/2-sqrt(2)*n_dist/a*Ts)*0.4; %input tube constant

max_size=Ts*N*a*lambda_tube+tube_size+terminal_set_size;
dist_set_func = @(theta_err,v,max_sin,max_cos) disturbance_set_function(theta_err,v,p,n_dist,n_direction,max_sin,max_cos); 




%% NTMPC - preperation 
% fmincon will find such input u that E*u<G. 
% If U is a vector of u of the horizon then A_fmincon*U<b_mincon
% U also has the output state at the end.
A_omega = [1,1;-1,1;1,-1;-1,-1];
b_omega = ones(4,1)*terminal_set_size;

E=[1/a,1/b;
	-1/a,1/b;
	1/a,-1/b;
	-1/a,-1/b];
G=ones(4,1)*lambda_tube;
options = optimoptions('fmincon','Algorithm','interior-point','Display','none','MaxFunctionEvaluations',10^6);
[A_fmincon,b_fmincon] = get_prediction_constraints(E,G,N);
A_fmincon=[A_fmincon,zeros(N*4,3);zeros(6,N*2),[1/tube_size,0,0;-1/tube_size,0,0;0,1/tube_size,0;0,-1/tube_size,0;0,0,1/pi;0,0,-1/pi]];
b_fmincon=[b_fmincon;ones(6,1)];
 
%% Parameters
% Robot's parameters
while true
    %% Simulation - preperation
    rng(pointer)

%     % Below variables are mostly used to store history data to plot them later.
    radius = rand()*max_size;
    direction = rand()*pi*2;
    state = [radius*cos(direction);radius*sin(direction);0];
    shape = get_tube_of_full_robot([0,0],p,state(1:2)');
    shape.minVRep();
    body_outside_of_environment_flag=false;
    for i=1:8
        if norm(shape.V(i,:))>max_size
            body_outside_of_environment_flag=true;
            break
        end
    end
    disp(pointer)
    if body_outside_of_environment_flag
        pointer=pointer+1;
        continue
    end

    save(filename,"dataset","pointer");
    %SDDTMPC simulation
    if mod(pointer,10)==1
        save(filename_helper,"dataset","pointer");
    end
    % Get warm start
    fmincon_state=state;
    cost_fun_fmincon = @(u) fmincon_cost_function(u,fmincon_state,N,Q,R,p,Ts);
    nonlinear_model =@(x,u) discrete_model(x',u,p,Ts);
    terminal_constraint_func= @(u) fmincon_termianl_constraint(u,fmincon_state,nonlinear_model,N,2,A_omega,b_omega,p+tube_size*2,max_size);
    for i=1:5
        [inputs,fval,exitflag,output] = fmincon(cost_fun_fmincon,zeros(2*N+3,1),A_fmincon,b_fmincon,[],[],[],[],terminal_constraint_func,options);
        if exitflag>0
            break
        end
        fmincon_state=fmincon_state*0.95;
        cost_fun_fmincon = @(u) fmincon_cost_function(u,fmincon_state,N,Q,R,p,Ts);
        terminal_constraint_func= @(u) fmincon_termianl_constraint(u,fmincon_state,nonlinear_model,N,2,A_omega,b_omega,p+tube_size*2,max_size);
    end

    if exitflag<1
        pointer=pointer+1;
        disp('NMPC failure')
        continue
    end
    %Solve SDDTMPC
    tic
    [inputs,fval,exitflag,message]  = SDDTMPC_solver(cost_fun_fmincon,inputs,state,...
       nonlinear_model,K,p,[-a;-b],[a;b],E,G/lambda_tube,tube_size,terminal_set_size,N,dist_set_func,150,Ts,max_size);
    if fval>10000
        pointer=pointer+1;
        disp('SDDTMPC failure')
        continue
    end
    toc
    mv=inputs(1:2);% Get nominal input

    error = -inputs(end-2:end);
    nominal_state=-error+state;
    % Store data and update state
    anciallary_u=rot(state(3),p)^(-1)*(rot(nominal_state(3),p)*mv-K*error(1:2)); %real input
    pointer = pointer+1;
    dataset(pointer,:)=[state(1:2)',anciallary_u',mv',error'];
    if pointer==1000000
        break
    end
        

end

end


function cost=myCostFunction(X,U,horizon,Q,R,p)  %x=[xe,ye,te], ue=[ve,we,xr,yr,tr]  par =[p,xd,yd,vr,wr]
    cost=0;
    for i = 1:horizon
        if i==horizon
            Q=0.5;
        end
        p_e= X(i,1:2)';
        cost=cost+p_e'*Q*p_e;
        u_e=[U(i,1);
            p*U(i,2)];
        cost = cost + u_e'*R*u_e;

    end

end


function cost=fmincon_cost_function(u,x,horizon,Q,R,p,Ts)
    x=x+u(end-2:end);
    All_U = reshape(u(1:end-1),(length(u)-3)/horizon,horizon+1)';
    X=zeros(horizon,length(x));
    for i=1:horizon
        x = discrete_model(x',All_U(i,:),p,Ts);
        X(i,:)=x;
    end
    cost=myCostFunction(X,All_U,horizon,Q,R,p);
end

function [c,ceq]=fmincon_termianl_constraint(inputs,nominal_state,model,horizon,number_of_inputs,A_omega,b_omega,robot_size,world_size)
    c=zeros(horizon+4,1);

    nominal_state=nominal_state+inputs(end-2:end);
    for i=1:horizon
        v=inputs((number_of_inputs*(i-1)+1):number_of_inputs*i)';
        nominal_state=model(nominal_state,v);
        dist =norm(nominal_state(1:2));
        c(i) = dist-world_size+robot_size;
    end
    c(end-3:end)= A_omega*nominal_state(1:2)-b_omega;
    ceq=0;
end
