import rclpy
from rclpy.node import Node
import math
import time


from geometry_msgs.msg import PoseStamped,Twist

import snntorch as  snn

import torch
import torch.nn as nn

import matplotlib.pyplot as plt
import numpy as np
import random
import math
import time

target = [0,0,0]
first_msg_flag=False
trajectory_flag=9
helper=0
path_list = [
[[0,0,0],[1,0,0],[0,1,0],[0,-1,0],[-1,0,0]],
[[0,0,0],[0,1,0],[-1,-1,0],[1,0,0]],
]
path_idx=0

class Net(torch.nn.Module):
    """Simple spiking neural network in snntorch."""

    def __init__(self, timesteps, hidden,input_size,output_size, learn_beta,learn_threshold,reset_mechanism):
        super().__init__()

        self.timesteps = timesteps # number of time steps to simulate the network
        self.hidden = hidden # number of hidden neurons
        #spike_grad = surrogate.LeakySpikeOperator() # surrogate gradient function
        out_features = input_size
        self.layers = nn.ModuleList()
        #parameters=nn.ParameterList()

        for hid in hidden:
            in_features=out_features
            out_features = hid
            beta_in = torch.ones(hid)*0.95
            fc = torch.nn.Linear(in_features=in_features, out_features=out_features)
            #parameters.append(fc.weight)
            #parameters.append(fc.bias)
            lif = snn.Leaky(beta=beta_in, learn_beta=learn_beta,learn_threshold=learn_threshold,reset_mechanism=reset_mechanism)
            self.layers.append(fc)
            self.layers.append(lif)
        beta_in = torch.ones(output_size)*0.95
        fc = torch.nn.Linear(in_features=hidden[-1], out_features=output_size)
        #parameters.append(fc.weight)
        #parameters.append(fc.bias)
        lif = snn.Leaky(beta=beta_in, learn_beta=learn_beta,learn_threshold=learn_threshold,reset_mechanism="none")
        self.layers.append(fc)
        self.layers.append(lif)
        #self.myparameters=parameters
        
        
    def forward(self, x):
        """Forward pass for several time steps."""

        # Initalize membrane potential
        mems = []
        for i,snn_layer in enumerate(self.layers):
            if i%2==1:
                mems.append(snn_layer.init_leaky())


        batch_size=x.size(dim=0)

        spike_history_in = torch.zeros(batch_size,sum(self.hidden))
        # Loop over
        for step in range(self.timesteps):
            spk = x
            idx=0
            #print(spk)
            for i in range(len(self.hidden)+1):
                cur = self.layers[2*i](spk)
                #print(cur.shape)
                spk,mem = self.layers[2*i+1](cur,mems[i])
                #print(mem.shape)
                #print(spk.shape)
                #print('----------')
                mems[i]=mem
                if i<len(self.hidden):
                    spike_history_in[:,idx:(idx+self.hidden[i])]=spike_history_in[:,idx:(idx+self.hidden[i])]+spk
                    idx=idx+self.hidden[i]
            
            

        #%return torch.mean(mem_4,1,True),spike_history_in
        return mems[-1],spike_history_in

def get_state_invariant(state):
    return np.matmul(state[0:2],np.matrix([[np.cos(state[2]),-np.sin(state[2])], [np.sin(state[2]), np.cos(state[2])]]))
    
def euler_from_quaternion(x, y, z, w):
        """
        Convert a quaternion into euler angles (roll, pitch, yaw)
        roll is rotation around x in radians (counterclockwise)
        pitch is rotation around y in radians (counterclockwise)
        yaw is rotation around z in radians (counterclockwise)
        """
        t0 = +2.0 * (w * x + y * z)
        t1 = +1.0 - 2.0 * (x * x + y * y)
        roll_x = math.atan2(t0, t1)
     
        t2 = +2.0 * (w * y - z * x)
        t2 = +1.0 if t2 > +1.0 else t2
        t2 = -1.0 if t2 < -1.0 else t2
        pitch_y = math.asin(t2)
     
        t3 = +2.0 * (w * z + x * y)
        t4 = +1.0 - 2.0 * (y * y + z * z)
        yaw_z = math.atan2(t3, t4)
     
        return roll_x, pitch_y, yaw_z # in radians
    
def scale_input(lin_vel,ang_vel,max_full_vel,radius):
    full_vel = abs(lin_vel)+abs(ang_vel)*radius
    if full_vel>max_full_vel:
        lin_vel=lin_vel*max_full_vel/full_vel
        ang_vel=ang_vel*max_full_vel/full_vel
    return np.concatenate([lin_vel, ang_vel])
    
def get_path():
    path=[]
    x = 0
    y_helper= 0
    for i in range(30):
        path.append([float(x),float(max(3*math.sin(y_helper),0))])
        x=1/3+x
        y_helper= y_helper+1/3
    return path


class MinimalPublisher(Node):

    def __init__(self):
        
        super().__init__('minimal_publisher')
        self.time=0
        #self.ref_input=[0.2,1]
        #self.ref_state=[0.01,0.01,0]
        self.path=get_path()
        self.iterator =0
        ### Very bad method but I do not have a better idea
        import __main__
        setattr(__main__, "Net", Net)

        self.linear_velocity_network = torch.load('linear_final.pt')
        self.angular_velocity_network = torch.load('angular_final.pt')

        self.max_lin_vel=0.46
        self.robot_radius = 0.4731/2;
        self.max_ang_vel=self.max_lin_vel/self.robot_radius
        self.reference = [0.5,0.5]
        self.publisher_ = self.create_publisher(Twist, 'cmd_vel', 10)
        #self.publisher_ref = self.create_publisher(Pose, 'ref_state', 10)
        self.timer_period = 0.3  # seconds
        self.timer = self.create_timer(self.timer_period, self.timer_callback)
        self.x = np.array([0,0,0])
        self.subscription = self.create_subscription(
            PoseStamped,
            'mocap',
            self.listener_callback,
            10)
        self.subscription  # prevent unused variable warning


    def timer_callback(self):
        global target, helper,path_list,path_idx
        #ref_msg = Pose()
        #print(self.path[self.iterator])
        #ref_msg.position.x = self.path[self.iterator][0]
        #ref_msg.position.y = self.path[self.iterator][1]
        #self.publisher_ref.publish(ref_msg)
        #self.ref_state[2] = self.ref_state[2] +self.ref_input[1]*self.timer_period
        #self.ref_state[0] = self.ref_state[0]+self.timer_period*(self.ref_input[0]*math.cos(self.ref_state[2])+self.ref_input[1]*math.sin(self.ref_state[2])*0.47/2)
        #self.ref_state[1] = self.ref_state[1]+self.timer_period*(self.ref_input[0]*math.sin(self.ref_state[2])+self.ref_input[1]*math.cos(self.ref_state[2])*0.47/2)
        #self.time = self.time + self.timer_period
        msg = Twist()
        #print('-----')
        invariant_state = get_state_invariant(self.x)
        print(self.x)
        #print(invariant_state)

        vel_lin = (self.linear_velocity_network(torch.FloatTensor(invariant_state))[0].detach().numpy()*self.max_lin_vel)[0]
	
        vel_ang = (self.angular_velocity_network(torch.FloatTensor(invariant_state))[0].detach().numpy()*self.max_ang_vel)[0]

        #print(vel_lin)
        #print(vel_ang)
        u = scale_input(vel_lin,vel_ang,self.max_lin_vel,self.robot_radius)
        #print(u)
        msg.linear.x=u[0].item()
        msg.angular.z=u[1].item()

        #aewqewq=5/0
        if first_msg_flag:
            self.publisher_.publish(msg)
            print(msg.linear.x)
            match trajectory_flag:
                case 1:
                    target[0] = target[0] + 0.03
                case 2:
                    target[0] = target[0] - 0.03
                case 3:
                    target[1] = target[1] + 0.03
                case 4:
                    target[0] = target[0] + 0.09
                case 5:
                    target[0] = target[0] - 0.09
                case 6:
                    target[0] = target[0] + 0.05
                    target[1] = target[1] + 0.05
                case 7:
                    helper = helper + 0.3
                    target[0] = target[0]+0.05*math.cos(helper*0.1)
                    target[1] = target[1]+0.05*math.sin(helper*0.1)
                case 8:
                    helper = helper + 0.3
                    target[0] = target[0] + 0.03
                    target[1] = target[1]+0.05*math.cos(helper*0.1)
                case _:
                    if first_msg_flag:
                        if ((self.x[0]*self.x[0]+self.x[1]*self.x[1])<0.01):
                            target[0] = target[0] + path_list[trajectory_flag-9][path_idx][0]
                            target[1] = target[1] + path_list[trajectory_flag-9][path_idx][1]
                            path_idx=path_idx+1
                            print(path_idx)
                            
                        
        #if self.iterator<29:
        #    if ((self.x[0])**2+(self.x[1])**2)<0.01:
        #        self.iterator = self.iterator+1

        
    def listener_callback(self,msg):
        #try:
        #    if ((msg.position.x-self.path[self.iterator][0])**2+(msg.position.y-self.path[self.iterator][1])**2)<0.04:
        #        self.iterator = self.iterator+1
        #        print(self.iterator)
        #        
        #except:
        #    pass
        global first_msg_flag, target
        x_dir,y_dir,z_dir = euler_from_quaternion(msg.pose.orientation.x, msg.pose.orientation.z, msg.pose.orientation.y, msg.pose.orientation.w)
        if not first_msg_flag:
            first_msg_flag=True
            target = [msg.pose.position.z+self.robot_radius*math.cos(z_dir),msg.pose.position.x+self.robot_radius*math.sin(z_dir),0]
            
        
        
        #self.x =[msg.position.x-self.path[self.iterator][0],msg.position.y-self.path[self.iterator][1],z_dir]
        self.x =[msg.pose.position.z-target[0]+self.robot_radius*math.cos(z_dir),msg.pose.position.x-target[1]+self.robot_radius*math.sin(z_dir),z_dir-target[2]]




def main(args=None):
    rclpy.init(args=args)

    minimal_publisher = MinimalPublisher()

    rclpy.spin(minimal_publisher)

    # Destroy the node explicitly
    # (optional - otherwise it will be done automatically
    # when the garbage collector destroys the node object)
    minimal_publisher.destroy_node()
    rclpy.shutdown()


if __name__ == '__main__':
    main()
