import snntorch as  snn

import torch
import torch.nn as nn

import matplotlib.pyplot as plt
import numpy as np
import random
import math
import time

import matlab.engine
import matlab

import sys

eng = matlab.engine.start_matlab()

class Net(torch.nn.Module):
    """Simple spiking neural network in snntorch."""

    def __init__(self, timesteps, hidden,input_size,output_size, learn_beta,learn_threshold,reset_mechanism):
        super().__init__()

        self.timesteps = timesteps # number of time steps to simulate the network
        self.hidden = hidden # number of hidden neurons
        #spike_grad = surrogate.LeakySpikeOperator() # surrogate gradient function
        out_features = input_size
        self.layers = nn.ModuleList()
        #parameters=nn.ParameterList()

        for hid in hidden:
            in_features=out_features
            out_features = hid
            beta_in = torch.ones(hid)*0.95
            fc = torch.nn.Linear(in_features=in_features, out_features=out_features)
            #parameters.append(fc.weight)
            #parameters.append(fc.bias)
            lif = snn.Leaky(beta=beta_in, learn_beta=learn_beta,learn_threshold=learn_threshold,reset_mechanism=reset_mechanism)
            self.layers.append(fc)
            self.layers.append(lif)
        beta_in = torch.ones(output_size)*0.95
        fc = torch.nn.Linear(in_features=hidden[-1], out_features=output_size)
        #parameters.append(fc.weight)
        #parameters.append(fc.bias)
        lif = snn.Leaky(beta=beta_in, learn_beta=learn_beta,learn_threshold=learn_threshold,reset_mechanism="none")
        self.layers.append(fc)
        self.layers.append(lif)
        #self.myparameters=parameters
        
        
    def forward(self, x):
        """Forward pass for several time steps."""

        # Initalize membrane potential
        mems = []
        for i,snn_layer in enumerate(self.layers):
            if i%2==1:
                mems.append(snn_layer.init_leaky())


        batch_size=x.size(dim=0)

        spike_history_in = torch.zeros(batch_size,sum(self.hidden))
        # Loop over
        for step in range(self.timesteps):
            spk = x
            idx=0
            #print(spk)
            for i in range(len(self.hidden)+1):
                cur = self.layers[2*i](spk)
                #print(cur.shape)
                spk,mem = self.layers[2*i+1](cur,mems[i])
                #print(mem.shape)
                #print(spk.shape)
                #print('----------')
                mems[i]=mem
                if i<len(self.hidden):
                    spike_history_in[:,idx:(idx+self.hidden[i])]=spike_history_in[:,idx:(idx+self.hidden[i])]+spk
                    idx=idx+self.hidden[i]
            
            

        #%return torch.mean(mem_4,1,True),spike_history_in
        return mems[-1],spike_history_in
        
def get_state_invariant(state):
    return np.matmul(state[0:2],np.matrix([[np.cos(state[2]),-np.sin(state[2])], [np.sin(state[2]), np.cos(state[2])]]))
    
    
def scale_input(lin_vel,ang_vel,max_full_vel,radius):
    full_vel = abs(lin_vel)+abs(ang_vel)*radius
    if full_vel>max_full_vel:
        lin_vel=lin_vel*max_full_vel/full_vel
        ang_vel=ang_vel*max_full_vel/full_vel
    return np.concatenate([lin_vel, ang_vel])
    
    
linear_velocity_network = torch.load('linear_final.pt')
angular_velocity_network = torch.load('angular_final.pt')	

max_lin_vel=0.46
robot_radius = 0.4731/2;
max_ang_vel=max_lin_vel/robot_radius


np.random.seed(int(sys.argv[1]))


i=int(sys.argv[2])
Ts=0.2
if i==0:
    state=np.array((1.6*np.random.random()-0.8,1.6*np.random.random()-0.8,2*math.pi*np.random.random()-math.pi))
else:
    file = open("raw_data/result_"+sys.argv[1]+'_'+str(i-1)+'.txt', "r")
    result = file.read() 
    file.close()
    if result !='Success':
        file = open("raw_data/result_"+sys.argv[1]+'_'+str(i)+'.txt', "w")
        file.write('Failure')
        file.close()
        sys.exit()
    file = open("raw_data/state_"+sys.argv[1]+'_'+str(i)+'.txt', "r")
    data_into_list = file.read().split(",") 
    state = np.array([float(i) for i in data_into_list])
    file.close()

invariant_state = get_state_invariant(state)
invariant_state_SDDTMPC =np.concatenate( [np.ravel(get_state_invariant(state)),np.array([0])])
vel_lin = (linear_velocity_network(torch.FloatTensor(invariant_state))[0].detach().numpy()*max_lin_vel)[0]
vel_ang = (angular_velocity_network(torch.FloatTensor(invariant_state))[0].detach().numpy()*max_ang_vel)[0]
input = scale_input(vel_lin,vel_ang,max_lin_vel,robot_radius)
file = open("raw_data/input_"+sys.argv[1]+'_'+str(i)+'.txt', "w")
file.write(str(input[0])+','+str(input[1]))
file.close()
if i==0:
    file = open("raw_data/state_"+sys.argv[1]+'_'+str(i)+'.txt', "w")
    file.write(str(state[0])+','+str(state[1])+','+str(state[2]))
    file.close()
